# /* Copyright (C) 2001
#  * Housemarque Oy
#  * http://www.housemarque.com
#  *
#  * Distributed under the Boost Software License, Version 1.0. (See
#  * accompanying file LICENSE_1_0.txt or copy at
#  * http://www.boost.org/LICENSE_1_0.txt)
#  */
#
# /* Revised by Paul Mensonides (2002) */
# /* Revised by Edward Diener (2020) */
#
# /* See http://www.boost.org for most recent version. */
#
# ifndef BOOST_PREPROCESSOR_LOGICAL_BOOL_512_HPP
# define BOOST_PREPROCESSOR_LOGICAL_BOOL_512_HPP
#
# define BOOST_PP_BOOL_257 1
# define BOOST_PP_BOOL_258 1
# define BOOST_PP_BOOL_259 1
# define BOOST_PP_BOOL_260 1
# define BOOST_PP_BOOL_261 1
# define BOOST_PP_BOOL_262 1
# define BOOST_PP_BOOL_263 1
# define BOOST_PP_BOOL_264 1
# define BOOST_PP_BOOL_265 1
# define BOOST_PP_BOOL_266 1
# define BOOST_PP_BOOL_267 1
# define BOOST_PP_BOOL_268 1
# define BOOST_PP_BOOL_269 1
# define BOOST_PP_BOOL_270 1
# define BOOST_PP_BOOL_271 1
# define BOOST_PP_BOOL_272 1
# define BOOST_PP_BOOL_273 1
# define BOOST_PP_BOOL_274 1
# define BOOST_PP_BOOL_275 1
# define BOOST_PP_BOOL_276 1
# define BOOST_PP_BOOL_277 1
# define BOOST_PP_BOOL_278 1
# define BOOST_PP_BOOL_279 1
# define BOOST_PP_BOOL_280 1
# define BOOST_PP_BOOL_281 1
# define BOOST_PP_BOOL_282 1
# define BOOST_PP_BOOL_283 1
# define BOOST_PP_BOOL_284 1
# define BOOST_PP_BOOL_285 1
# define BOOST_PP_BOOL_286 1
# define BOOST_PP_BOOL_287 1
# define BOOST_PP_BOOL_288 1
# define BOOST_PP_BOOL_289 1
# define BOOST_PP_BOOL_290 1
# define BOOST_PP_BOOL_291 1
# define BOOST_PP_BOOL_292 1
# define BOOST_PP_BOOL_293 1
# define BOOST_PP_BOOL_294 1
# define BOOST_PP_BOOL_295 1
# define BOOST_PP_BOOL_296 1
# define BOOST_PP_BOOL_297 1
# define BOOST_PP_BOOL_298 1
# define BOOST_PP_BOOL_299 1
# define BOOST_PP_BOOL_300 1
# define BOOST_PP_BOOL_301 1
# define BOOST_PP_BOOL_302 1
# define BOOST_PP_BOOL_303 1
# define BOOST_PP_BOOL_304 1
# define BOOST_PP_BOOL_305 1
# define BOOST_PP_BOOL_306 1
# define BOOST_PP_BOOL_307 1
# define BOOST_PP_BOOL_308 1
# define BOOST_PP_BOOL_309 1
# define BOOST_PP_BOOL_310 1
# define BOOST_PP_BOOL_311 1
# define BOOST_PP_BOOL_312 1
# define BOOST_PP_BOOL_313 1
# define BOOST_PP_BOOL_314 1
# define BOOST_PP_BOOL_315 1
# define BOOST_PP_BOOL_316 1
# define BOOST_PP_BOOL_317 1
# define BOOST_PP_BOOL_318 1
# define BOOST_PP_BOOL_319 1
# define BOOST_PP_BOOL_320 1
# define BOOST_PP_BOOL_321 1
# define BOOST_PP_BOOL_322 1
# define BOOST_PP_BOOL_323 1
# define BOOST_PP_BOOL_324 1
# define BOOST_PP_BOOL_325 1
# define BOOST_PP_BOOL_326 1
# define BOOST_PP_BOOL_327 1
# define BOOST_PP_BOOL_328 1
# define BOOST_PP_BOOL_329 1
# define BOOST_PP_BOOL_330 1
# define BOOST_PP_BOOL_331 1
# define BOOST_PP_BOOL_332 1
# define BOOST_PP_BOOL_333 1
# define BOOST_PP_BOOL_334 1
# define BOOST_PP_BOOL_335 1
# define BOOST_PP_BOOL_336 1
# define BOOST_PP_BOOL_337 1
# define BOOST_PP_BOOL_338 1
# define BOOST_PP_BOOL_339 1
# define BOOST_PP_BOOL_340 1
# define BOOST_PP_BOOL_341 1
# define BOOST_PP_BOOL_342 1
# define BOOST_PP_BOOL_343 1
# define BOOST_PP_BOOL_344 1
# define BOOST_PP_BOOL_345 1
# define BOOST_PP_BOOL_346 1
# define BOOST_PP_BOOL_347 1
# define BOOST_PP_BOOL_348 1
# define BOOST_PP_BOOL_349 1
# define BOOST_PP_BOOL_350 1
# define BOOST_PP_BOOL_351 1
# define BOOST_PP_BOOL_352 1
# define BOOST_PP_BOOL_353 1
# define BOOST_PP_BOOL_354 1
# define BOOST_PP_BOOL_355 1
# define BOOST_PP_BOOL_356 1
# define BOOST_PP_BOOL_357 1
# define BOOST_PP_BOOL_358 1
# define BOOST_PP_BOOL_359 1
# define BOOST_PP_BOOL_360 1
# define BOOST_PP_BOOL_361 1
# define BOOST_PP_BOOL_362 1
# define BOOST_PP_BOOL_363 1
# define BOOST_PP_BOOL_364 1
# define BOOST_PP_BOOL_365 1
# define BOOST_PP_BOOL_366 1
# define BOOST_PP_BOOL_367 1
# define BOOST_PP_BOOL_368 1
# define BOOST_PP_BOOL_369 1
# define BOOST_PP_BOOL_370 1
# define BOOST_PP_BOOL_371 1
# define BOOST_PP_BOOL_372 1
# define BOOST_PP_BOOL_373 1
# define BOOST_PP_BOOL_374 1
# define BOOST_PP_BOOL_375 1
# define BOOST_PP_BOOL_376 1
# define BOOST_PP_BOOL_377 1
# define BOOST_PP_BOOL_378 1
# define BOOST_PP_BOOL_379 1
# define BOOST_PP_BOOL_380 1
# define BOOST_PP_BOOL_381 1
# define BOOST_PP_BOOL_382 1
# define BOOST_PP_BOOL_383 1
# define BOOST_PP_BOOL_384 1
# define BOOST_PP_BOOL_385 1
# define BOOST_PP_BOOL_386 1
# define BOOST_PP_BOOL_387 1
# define BOOST_PP_BOOL_388 1
# define BOOST_PP_BOOL_389 1
# define BOOST_PP_BOOL_390 1
# define BOOST_PP_BOOL_391 1
# define BOOST_PP_BOOL_392 1
# define BOOST_PP_BOOL_393 1
# define BOOST_PP_BOOL_394 1
# define BOOST_PP_BOOL_395 1
# define BOOST_PP_BOOL_396 1
# define BOOST_PP_BOOL_397 1
# define BOOST_PP_BOOL_398 1
# define BOOST_PP_BOOL_399 1
# define BOOST_PP_BOOL_400 1
# define BOOST_PP_BOOL_401 1
# define BOOST_PP_BOOL_402 1
# define BOOST_PP_BOOL_403 1
# define BOOST_PP_BOOL_404 1
# define BOOST_PP_BOOL_405 1
# define BOOST_PP_BOOL_406 1
# define BOOST_PP_BOOL_407 1
# define BOOST_PP_BOOL_408 1
# define BOOST_PP_BOOL_409 1
# define BOOST_PP_BOOL_410 1
# define BOOST_PP_BOOL_411 1
# define BOOST_PP_BOOL_412 1
# define BOOST_PP_BOOL_413 1
# define BOOST_PP_BOOL_414 1
# define BOOST_PP_BOOL_415 1
# define BOOST_PP_BOOL_416 1
# define BOOST_PP_BOOL_417 1
# define BOOST_PP_BOOL_418 1
# define BOOST_PP_BOOL_419 1
# define BOOST_PP_BOOL_420 1
# define BOOST_PP_BOOL_421 1
# define BOOST_PP_BOOL_422 1
# define BOOST_PP_BOOL_423 1
# define BOOST_PP_BOOL_424 1
# define BOOST_PP_BOOL_425 1
# define BOOST_PP_BOOL_426 1
# define BOOST_PP_BOOL_427 1
# define BOOST_PP_BOOL_428 1
# define BOOST_PP_BOOL_429 1
# define BOOST_PP_BOOL_430 1
# define BOOST_PP_BOOL_431 1
# define BOOST_PP_BOOL_432 1
# define BOOST_PP_BOOL_433 1
# define BOOST_PP_BOOL_434 1
# define BOOST_PP_BOOL_435 1
# define BOOST_PP_BOOL_436 1
# define BOOST_PP_BOOL_437 1
# define BOOST_PP_BOOL_438 1
# define BOOST_PP_BOOL_439 1
# define BOOST_PP_BOOL_440 1
# define BOOST_PP_BOOL_441 1
# define BOOST_PP_BOOL_442 1
# define BOOST_PP_BOOL_443 1
# define BOOST_PP_BOOL_444 1
# define BOOST_PP_BOOL_445 1
# define BOOST_PP_BOOL_446 1
# define BOOST_PP_BOOL_447 1
# define BOOST_PP_BOOL_448 1
# define BOOST_PP_BOOL_449 1
# define BOOST_PP_BOOL_450 1
# define BOOST_PP_BOOL_451 1
# define BOOST_PP_BOOL_452 1
# define BOOST_PP_BOOL_453 1
# define BOOST_PP_BOOL_454 1
# define BOOST_PP_BOOL_455 1
# define BOOST_PP_BOOL_456 1
# define BOOST_PP_BOOL_457 1
# define BOOST_PP_BOOL_458 1
# define BOOST_PP_BOOL_459 1
# define BOOST_PP_BOOL_460 1
# define BOOST_PP_BOOL_461 1
# define BOOST_PP_BOOL_462 1
# define BOOST_PP_BOOL_463 1
# define BOOST_PP_BOOL_464 1
# define BOOST_PP_BOOL_465 1
# define BOOST_PP_BOOL_466 1
# define BOOST_PP_BOOL_467 1
# define BOOST_PP_BOOL_468 1
# define BOOST_PP_BOOL_469 1
# define BOOST_PP_BOOL_470 1
# define BOOST_PP_BOOL_471 1
# define BOOST_PP_BOOL_472 1
# define BOOST_PP_BOOL_473 1
# define BOOST_PP_BOOL_474 1
# define BOOST_PP_BOOL_475 1
# define BOOST_PP_BOOL_476 1
# define BOOST_PP_BOOL_477 1
# define BOOST_PP_BOOL_478 1
# define BOOST_PP_BOOL_479 1
# define BOOST_PP_BOOL_480 1
# define BOOST_PP_BOOL_481 1
# define BOOST_PP_BOOL_482 1
# define BOOST_PP_BOOL_483 1
# define BOOST_PP_BOOL_484 1
# define BOOST_PP_BOOL_485 1
# define BOOST_PP_BOOL_486 1
# define BOOST_PP_BOOL_487 1
# define BOOST_PP_BOOL_488 1
# define BOOST_PP_BOOL_489 1
# define BOOST_PP_BOOL_490 1
# define BOOST_PP_BOOL_491 1
# define BOOST_PP_BOOL_492 1
# define BOOST_PP_BOOL_493 1
# define BOOST_PP_BOOL_494 1
# define BOOST_PP_BOOL_495 1
# define BOOST_PP_BOOL_496 1
# define BOOST_PP_BOOL_497 1
# define BOOST_PP_BOOL_498 1
# define BOOST_PP_BOOL_499 1
# define BOOST_PP_BOOL_500 1
# define BOOST_PP_BOOL_501 1
# define BOOST_PP_BOOL_502 1
# define BOOST_PP_BOOL_503 1
# define BOOST_PP_BOOL_504 1
# define BOOST_PP_BOOL_505 1
# define BOOST_PP_BOOL_506 1
# define BOOST_PP_BOOL_507 1
# define BOOST_PP_BOOL_508 1
# define BOOST_PP_BOOL_509 1
# define BOOST_PP_BOOL_510 1
# define BOOST_PP_BOOL_511 1
# define BOOST_PP_BOOL_512 1
#
# endif
