#ifndef BOOST_UUID_HPP_INCLUDED
#define BOOST_UUID_HPP_INCLUDED

// Copyright 2024 Peter Dimov
// Distributed under the Boost Software License, Version 1.0.
// https://www.boost.org/LICENSE_1_0.txt

#include <boost/uuid/uuid.hpp>
#include <boost/uuid/uuid_io.hpp>
#include <boost/uuid/uuid_generators.hpp>

#endif // #ifndef BOOST_UUID_HPP_INCLUDED
