/* Copyright (c) 2018-2024 Marcelo Zimbres Silva (mzimbres@gmail.com)
 *
 * Distributed under the Boost Software License, Version 1.0. (See
 * accompanying file LICENSE.txt)
 */

#ifndef BOOST_REDIS_HPP
#define BOOST_REDIS_HPP

#include <boost/redis/config.hpp>
#include <boost/redis/connection.hpp>
#include <boost/redis/error.hpp>
#include <boost/redis/ignore.hpp>
#include <boost/redis/logger.hpp>
#include <boost/redis/request.hpp>
#include <boost/redis/response.hpp>

/** @defgroup high-level-api Reference
 *
 *  This page contains the documentation of the Aedis high-level API.
 */

/** @defgroup low-level-api Reference
 *
 *  This page contains the documentation of the Aedis low-level API.
 */

#endif  // BOOST_REDIS_HPP
