#ifndef BOOST_UUID_BASIC_NAME_GENERATOR_HPP_INCLUDED
#define BOOST_UUID_BASIC_NAME_GENERATOR_HPP_INCLUDED

// Copyright 2024 Peter Dimov
// Distributed under the Boost Software License, Version 1.0.
// https://www.boost.org/LICENSE_1_0.txt

// This header is only provided for compatibility with
// Boost release 1.85 and earlier.

#include <boost/uuid/detail/basic_name_generator.hpp>

namespace boost {
namespace uuids {

// Only provided for compatibility with 1.85
using detail::basic_name_generator;

} // uuids
} // boost

#endif // BOOST_UUID_BASIC_NAME_GENERATOR_HPP_INCLUDED
