// (C) Copyright Andrey Semashev 2025.
// Distributed under the Boost Software License, Version 1.0. (See
// accompanying file LICENSE_1_0.txt or copy at
// http://www.boost.org/LICENSE_1_0.txt)

#ifndef BOOST_SHARED_CONTAINER_ITERATOR_HPP
#define BOOST_SHARED_CONTAINER_ITERATOR_HPP

// This is a deprecated header left for backward compatibility.
// Please use <boost/iterator/shared_container_iterator.hpp> instead.

#include <boost/iterator/shared_container_iterator.hpp>

#endif // BOOST_SHARED_CONTAINER_ITERATOR_HPP
